﻿Imports System.Net
Imports PLCCommLib

''' <summary>PLC通信のサンプル</summary>
Public Class SampleForm

#Region "フィールド"

    ''' <summary>通信処理クラスのインスタンス</summary>
    Private _PLC As PLCComm = Nothing

#End Region

#Region "コンストラクタ"

    ''' <summary>既定のコンストラクタ</summary>
    Public Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。

        ' コントロールの有効・無効設定
        btnOpen.Enabled = True
        btnClose.Enabled = False
        grpParameter.Enabled = True
        grpBlock.Enabled = False
        grpBit.Enabled = False

        ' 設定の初期値
        cmbVender.SelectedIndex = 0     ' 三菱
        cmbProtocol.SelectedIndex = 0   ' MCプロトコル(3E)
        rdoTCP.Checked = True
        rdoBIN.Checked = True

    End Sub

#End Region

#Region "プロパティ"

#End Region

#Region "メソッド(Dummy)"

    ''' <summary>このままでは実行されないダミーのサンプルコード</summary>
    ''' <remarks>ランダム読出</remarks>
    Private Sub DummyMethod_Random()
        Dim dev As List(Of String) = New List(Of String)
        Dim size As Integer
        Dim dat() As Short
        Dim ret As Integer

        ' 通信クラスのインスタンス生成前は不可
        If _PLC Is Nothing Then Return

        ' 通信回線オープンしないと通信は不可
        If _PLC.IsOpened() = False Then Return


        ' 読出サイズ
        ' ※ワード単位の点数
        size = 4
        ReDim dat(size - 1)

        ' デバイス
        ' ※PLCに合わせたワードデバイスを指定
        Select Case _PLC.VenderName
            Case PLCComm.VenderNames.MITSUBISHI ' 三菱
                dev.Add("X0")       ' ビットのデバイス番号は16の倍数を推奨
                dev.Add("M0")       ' ビットのデバイス番号は16の倍数を推奨
                dev.Add("D1000")
                dev.Add("R5000")
            Case PLCComm.VenderNames.OMRON      ' OMRON
                dev.Add("0")        ' ワード形式で指定(.0～.15を付けない)
                dev.Add("W0")       ' ワード形式で指定(.0～.15を付けない)
                dev.Add("D1000")
                dev.Add("E5000")
            Case PLCComm.VenderNames.KEYENCE    ' KEYENCE
                dev.Add("R000")     ' ビットのデバイス番号下2桁(00～15のビット指定)は00を推奨
                dev.Add("MR000")    ' ビットのデバイス番号の下2桁(00～15のビット指定)は00を推奨
                dev.Add("DM1000")
                dev.Add("FM5000")
            Case Else
                Return
        End Select

        ' ランダム読出
        ret = _PLC.ReadDeviceRandom(dev.ToArray(), size, dat)
        If ret = PLCComm.RET_COMPLETED Then
            ' 読出成功 → 戻り値が0
        Else
            ' 読出失敗 → 戻り値が0以外
            ' 0x??000000の場合は通信ライブラリが返すエラーコード
            ' 0x0000????の場合は通信プロトコル上でPLC側が返すエラーコード
            ' ⇒エラーコードの詳細はプロトコルかEthernetユニットかCPUのいずれかのマニュアルに記載されているハズ
            Console.WriteLine("ReadDeviceRandom Fail. 0x" & ret.ToString("X8"))
        End If

    End Sub

    ''' <summary>このままでは実行されないダミーのサンプルコード</summary>
    ''' <remarks>ブロック読出→データ変換</remarks>
    Private Sub DummyMethod_Block()
        Dim dev As String
        Dim size As Integer
        Dim dat() As Short
        Dim ret As Integer

        ' 通信クラスのインスタンス生成前は不可
        If _PLC Is Nothing Then Return

        ' 通信回線オープンしないと通信は不可
        If _PLC.IsOpened() = False Then Return


        ' サイズ
        size = 2000
        ReDim dat(size - 1)

        ' デバイス
        ' ※PLCに合わせたワードデバイスを指定
        Select Case _PLC.VenderName
            Case PLCComm.VenderNames.MITSUBISHI ' 三菱
                'dev = "X0"      ' ビットのデバイス番号は16の倍数を推奨
                dev = "D0"
            Case PLCComm.VenderNames.OMRON      ' OMRON
                'dev = "0"       ' ワード形式で指定(.0～.15を付けない)
                dev = "D0"
            Case PLCComm.VenderNames.KEYENCE    ' KEYENCE
                'dev = "R000"    ' ビットのデバイス番号下2桁(00～15のビット指定)は00を推奨
                dev = "DM0"
            Case Else
                Return
        End Select

        ' ブロック読出
        ret = _PLC.ReadDeviceBlock(dev, size, dat)
        If ret = PLCComm.RET_COMPLETED Then
            ' 読出成功 → 戻り値が0
            Dim dword As Integer
            Dim flt1 As Single
            Dim ascii As String

            ' D100/D101のダブルワードに変換
            dword = DataConv.Word2DoubleWord(dat(100), dat(101))

            ' D500/D501の単精度浮動小数に変換
            flt1 = DataConv.Word2Single(dat(500), dat(501))

            ' D1000～のAscii文字列に変換
            ascii = DataConv.Word2Ascii(dat, False, 1000)   ' 下位Byte→上位Byteで格納される場合
            'ascii = DataConv.Word2Ascii(dat, True, 1000)    ' 上位Byte→下位Byteで格納される場合
            ' ※PLC内で通常の文字列処理をする場合
            ' 　　三菱:    下位→上位
            ' 　　OMRON:   上位→下位
            ' 　　KEYENCE: 上位→下位
            ' 　で格納される。ただし上位通信との都合などにより逆にして使う場合もあり得る。

        Else
            ' 読出失敗 → 戻り値が0以外
            ' 0x??000000の場合は通信ライブラリが返すエラーコード
            ' 0x0000????の場合は通信プロトコル上でPLC側が返すエラーコード
            ' ⇒エラーコードの詳細はプロトコルかEthernetユニットかCPUのいずれかのマニュアルに記載されているハズ
            Console.WriteLine("ReadDeviceBlock Fail. 0x" & ret.ToString("X8"))
        End If

    End Sub

    ''' <summary>このままでは実行されないダミーのサンプルコード</summary>
    ''' <remarks>ビット処理</remarks>
    Private Sub DummyMethod_Bit()
        Dim dev1, dev2 As String
        Dim ret As Integer

        ' 通信クラスのインスタンス生成前は不可
        If _PLC Is Nothing Then Return

        ' 通信回線オープンしないと通信は不可
        If _PLC.IsOpened() = False Then Return


        ' デバイス
        ' ※PLCに合わせたワードデバイスを指定
        ' ※ワードデバイスのビット指定も可(タイマ・カウンタ現在値やインデックスは除く)
        ' 　ただしMCプロトコルや上位リンクではプロトコル的にサポートされていないため
        ' 　PLC側の処理内容によっては予期せぬ不具合を招く可能性がある
        Select Case _PLC.VenderName
            Case PLCComm.VenderNames.MITSUBISHI ' 三菱
                dev1 = "Y0"
                dev2 = "D500.A"     ' 三菱はビット番号を16進数で指定
            Case PLCComm.VenderNames.OMRON      ' OMRON
                dev1 = "0.00"
                dev2 = "D500.10"    ' OMRONはビット番号を10進数で指定
            Case PLCComm.VenderNames.KEYENCE    ' KEYENCE
                dev1 = "R000"
                dev2 = "DM500.10"   ' KEYENCEはビット番号を10進数で指定
            Case Else
                Return
        End Select

        ' ビットSET(1)
        ret = _PLC.SetBit(dev1)

        ' ビットRESET(0)
        ret = _PLC.ResetBit(dev1)

        ' ビット反転
        ret = _PLC.InverseBit(dev2)
        ' ※通信ライブラリ側で読出→論理演算→SET or RESETの順に処理
        ' 　PLC側の処理内容によっては正常に動作しない可能性がある

    End Sub

#End Region

#Region "イベント処理"

    ''' <summary>通信回線接続ボタンクリック</summary>
    ''' <remarks>通信設定チェック→通信クラスのインスタンス生成→オープン→コントロールの有効・無効設定</remarks>
    Private Sub btnOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpen.Click
        Dim vnd As PLCComm.VenderNames = CType(cmbVender.SelectedIndex, PLCComm.VenderNames)
        Dim pro As PLCComm.ProtocolNames = CType(cmbProtocol.SelectedIndex, PLCComm.ProtocolNames)
        Dim ip As String
        Dim ipadr As IPAddress = New IPAddress(0)
        Dim port As Integer

        ' ベンダとプロトコルの組合せ
        Select Case vnd
            Case PLCComm.VenderNames.MITSUBISHI
                If pro <> PLCComm.ProtocolNames.MC3E Then Return

            Case PLCComm.VenderNames.OMRON
                If pro <> PLCComm.ProtocolNames.FINS Then Return

            Case PLCComm.VenderNames.KEYENCE
                If pro <> PLCComm.ProtocolNames.HiLink AndAlso pro <> PLCComm.ProtocolNames.MC3E Then Return

            Case Else
                Return
        End Select

        ' IPアドレス
        ip = txtIP0.Text & "." & txtIP1.Text & "." & txtIP2.Text & "." & txtIP3.Text
        If IPAddress.TryParse(ip, ipadr) = False Then Return

        ' 通信ポート
        If Integer.TryParse(txtPort.Text, port) = False Then Return



        ' 通信処理のインスタンスを生成
        Select Case vnd
            Case PLCComm.VenderNames.MITSUBISHI
                _PLC = New PLC_MC()

            Case PLCComm.VenderNames.OMRON
                _PLC = New PLC_Fins()

            Case PLCComm.VenderNames.KEYENCE
                If pro = PLCComm.ProtocolNames.HiLink Then
                    _PLC = New PLC_HiLink()
                ElseIf pro = PLCComm.ProtocolNames.MC3E Then
                    _PLC = New PLC_MCKV()
                Else
                    Return
                End If

            Case Else
                Return
        End Select

        ' 設定 ※必要に応じて変更
        '_PLC.Timeout = 3000
        _PLC.IsUdp = rdoUDP.Checked
        _PLC.IsAscii = rdoASC.Checked
        _PLC.IPAddress = ip
        _PLC.PortNumber = port
        '' ↓MCプロトコル用↓
        'If _PLC.ProtocolName = PLCComm.ProtocolNames.MC3E Then
        '    _PLC.NetworkNumber = 0
        '    _PLC.PCNumber = &HFF
        '    _PLC.ReqUnitIONumber = &H3FF
        '    _PLC.ReqUnitStnNumber = 0
        '    _PLC.CPUCheckTime = 8
        'End If
        '' ↓FINS用↓
        'If _PLC.ProtocolName = PLCComm.ProtocolNames.FINS Then
        '    _PLC.DNA = 0
        '    _PLC.DA1 = 0
        '    _PLC.DA2 = 0
        'End If



        ' 回線オープン
        Dim ret As Integer = _PLC.Open()
        If ret = PLCComm.RET_COMPLETED Then
            ' 成功。
            btnOpen.Enabled = False
            btnOpen.BackColor = Color.Lime
            btnClose.Enabled = True
            grpParameter.Enabled = False
            grpBlock.Enabled = True
            grpBit.Enabled = True

            ' デバイステキストボックス用のツールチップ
            ToolTip1.RemoveAll()
            Dim tip As System.Text.StringBuilder = New System.Text.StringBuilder(1024)
            Dim tipbit As System.Text.StringBuilder = New System.Text.StringBuilder(1024)
            Dim info() As PLCComm.DeviceInformation
            info = _PLC.GetAvailableDeviceInfo()
            For i = 0 To info.Length - 1
                tip.AppendLine(info(i).Name & ": " & info(i).Description)
                If info(i).fBit OrElse info(i).fWordBit Then
                    tipbit.AppendLine(info(i).Name & ": " & info(i).Description)
                End If
            Next
            ToolTip1.SetToolTip(txtDevBlock, tip.ToString())
            ToolTip1.SetToolTip(txtDevBit, tipbit.ToString())

        Else
            ' 失敗。
            Console.WriteLine("Open Fail... 0x" & ret.ToString("X8"))
        End If

    End Sub

    ''' <summary>通信回線切断ボタンクリック</summary>
    ''' <remarks>クローズ→コントロールの有効・無効設定</remarks>
    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        If _PLC Is Nothing Then Return

        _PLC.Close()
        _PLC = Nothing

        btnOpen.Enabled = True
        btnOpen.BackColor = SystemColors.Control
        btnClose.Enabled = False
        grpParameter.Enabled = True
        grpBlock.Enabled = False
        grpBit.Enabled = False

    End Sub

    ''' <summary>ブロック読出ボタンクリック</summary>
    ''' <remarks>指定の先頭デバイス(ワード)から指定サイズ(点数)の読出→データ表示</remarks>
    Private Sub btnReadBlock_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReadBlock.Click
        Dim dev As String
        Dim size As Integer
        Dim dat() As Short
        Dim ret As Integer

        ' チェック
        If _PLC Is Nothing Then Return
        dev = _PLC.RemakeDevice(txtDevBlock.Text)
        If String.IsNullOrEmpty(dev) Then Return
        'If _PLC.IsWordAvailable(dev) = False Then Return ' ←コレでチェックするとビットデバイス指定時に制約が増える
        If Integer.TryParse(txtSize.Text, size) = False Then Return
        If size < 1 Then Return

        ' 読出
        ReDim dat(size - 1)
        ret = _PLC.ReadDeviceBlock(dev, size, dat)

        If ret = PLCComm.RET_COMPLETED Then
            ' 読出成功
            Dim txt As System.Text.StringBuilder = New System.Text.StringBuilder(2048)
            txt.AppendLine(dev & ", " & size.ToString())
            txt.AppendLine("Read Time = " & _PLC.GetLastProcessingTime.ToString() & "ms")
            For i = 0 To size - 1
                If i >= 50 Then
                    txt.AppendLine("...")
                    Exit For
                End If
                txt.AppendLine("[" & i.ToString("D2") & "] = " & dat(i).ToString())
            Next
            txtReadData.Text = txt.ToString()

        Else
            ' 読出失敗
            txtReadData.Text = "Read Fail..." & vbCrLf & "0x" & ret.ToString("X8")
        End If

    End Sub

    ''' <summary>ブロック書込ボタンクリック</summary>
    ''' <remarks>指定の先頭デバイス(ワード)から指定サイズ(点数)の書込</remarks>
    Private Sub btnWriteBlock_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnWriteBlock.Click
        Dim dev As String
        Dim size As Integer
        Dim dat() As Short
        Dim ret As Integer

        ' チェック
        If _PLC Is Nothing Then Return
        dev = _PLC.RemakeDevice(txtDevBlock.Text)
        If String.IsNullOrEmpty(dev) Then Return
        'If _PLC.IsWordAvailable(dev) = False Then Return ' ←コレでチェックするとビットデバイス指定時に制約が増える
        If Integer.TryParse(txtSize.Text, size) = False Then Return
        If size < 1 Then Return

        ' 書込
        ReDim dat(size - 1) ' 配列確保で要素は全て0
        ret = _PLC.WriteDeviceBlock(dev, size, dat)

        If ret <> PLCComm.RET_COMPLETED Then
            Console.WriteLine("Write Fail... 0x" & ret.ToString("X8"))
        End If

    End Sub

    ''' <summary>SETボタンクリック</summary>
    ''' <remarks>指定のビットデバイス(ワードデバイスのビットも可)をSET</remarks>
    Private Sub btnBitSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBitSet.Click
        Dim dev As String
        Dim ret As Integer

        ' チェック
        If _PLC Is Nothing Then Return
        dev = _PLC.RemakeDevice(txtDevBit.Text)
        If dev = "" Then Return
        If _PLC.IsBitDevice(dev) = False Then Return

        ' ビットセット
        ret = _PLC.SetBit(dev)
        If ret <> PLCComm.RET_COMPLETED Then
            Console.WriteLine("SET Fail... 0x" & ret.ToString("X8"))
        End If

    End Sub

    ''' <summary>RSTボタンクリック</summary>
    ''' <remarks>指定のビットデバイス(ワードデバイスのビットも可)をRESET</remarks>
    Private Sub btnBitReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBitReset.Click
        Dim dev As String
        Dim ret As Integer

        ' チェック
        If _PLC Is Nothing Then Return
        dev = _PLC.RemakeDevice(txtDevBit.Text)
        If dev = "" Then Return
        If _PLC.IsBitDevice(dev) = False Then Return

        ' ビットリセット
        ret = _PLC.ResetBit(dev)
        If ret <> PLCComm.RET_COMPLETED Then
            Console.WriteLine("RST Fail... 0x" & ret.ToString("X8"))
        End If

    End Sub

    ''' <summary>反転ボタンクリック</summary>
    ''' <remarks>指定のビットデバイス(ワードデバイスのビットも可)を反転</remarks>
    Private Sub btnBitInverse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBitInverse.Click
        Dim dev As String
        Dim ret As Integer

        ' チェック
        If _PLC Is Nothing Then Return
        dev = _PLC.RemakeDevice(txtDevBit.Text)
        If dev = "" Then Return
        If _PLC.IsBitDevice(dev) = False Then Return

        ' ビット反転
        ret = _PLC.InverseBit(dev)
        If ret <> PLCComm.RET_COMPLETED Then
            Console.WriteLine("INV Fail... 0x" & ret.ToString("X8"))
        End If

    End Sub



    ''' <summary>フォームクローズ</summary>
    ''' <remarks>通信回線切断</remarks>
    Private Sub SampleForm_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If _PLC IsNot Nothing Then _PLC.Close()
    End Sub



    ''' <summary>デバイス入力テキストボックスのマウスホイール</summary>
    ''' <remarks>ホイールに合わせてデバイスをINC・DEC</remarks>
    Private Sub txtDevBlock_MouseWheel(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles txtDevBlock.MouseWheel, txtDevBit.MouseWheel
        Dim txt As TextBox = CType(sender, TextBox)
        Dim dev As String = txt.Text
        Dim div As Integer = e.Delta \ SystemInformation.MouseWheelScrollDelta ' ← 通常±3の範囲になるハズ

        ' デバイスと認識できないものは処理しない
        If String.IsNullOrEmpty(txtDevBlock.Text) Then Return
        If txt.Name = "txtDevBlock" Then
            If _PLC.IsDevice(dev) = False Then Return
        Else
            If _PLC.IsBitDevice(dev) = False Then Return
        End If

        ' ホイール回転分INC/DEC
        dev = _PLC.RemakeDevice(dev, div)
        If String.IsNullOrEmpty(dev) Then Return

        ' カーソル位置を合わせてTextを変更
        Dim selLast As Integer = txt.Text.Length - txt.SelectionStart
        txt.Text = dev
        If txt.Text.Length > selLast Then txt.SelectionStart = txt.Text.Length - selLast

    End Sub

    ''' <summary>デバイス入力テキストボックスのキーダウン</summary>
    ''' <remarks>Ctrl+Shift+Up/DownでデバイスをINC・DEC</remarks>
    Private Sub txtDevBlock_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtDevBlock.KeyDown, txtDevBit.KeyDown
        Dim txt As TextBox = CType(sender, TextBox)
        Dim dev As String = txt.Text
        Dim div As Integer

        ' デバイスと認識できないものは処理しない
        If String.IsNullOrEmpty(dev) Then Return
        If txt.Name = "txtDevBlock" Then
            If _PLC.IsDevice(dev) = False Then Return
        Else
            If _PLC.IsBitDevice(dev) = False Then Return
        End If

        ' Up or DownキーでINC or DEC
        If e.Control AndAlso e.Shift Then
            If e.KeyCode = Keys.Up Then
                div = 1
            ElseIf e.KeyCode = Keys.Down Then
                div = -1
            Else
                Return
            End If
        Else
            Return
        End If
        dev = _PLC.RemakeDevice(dev, div)
        If String.IsNullOrEmpty(dev) Then Return

        ' カーソル位置を合わせてTextを変更
        Dim selLast As Integer = txt.Text.Length - txt.SelectionStart
        txt.Text = dev
        If txt.Text.Length > selLast Then txt.SelectionStart = txt.Text.Length - selLast

        e.Handled = True
    End Sub

#End Region

End Class
